@@ -12,7 +12,7 @@ try:
 except ImportError:
     from django.utils.datastructures import SortedDict as OrderedDict
 
-from django.core.exceptions import ImproperlyConfigured
+from django.core.exceptions import ImproperlyConfigured, FieldDoesNotExist
 from django.utils import six
 
 from haystack import fields as haystack_fields
@@ -21,7 +21,6 @@ from haystack.utils import Highlighter
 
 from rest_framework import serializers
 from rest_framework.fields import empty
-from rest_framework.pagination import _get_count
 from rest_framework.utils.field_mapping import ClassLookupDict, get_field_kwargs
 
 from drf_haystack.fields import (
@@ -128,9 +127,10 @@ class HaystackSerializer(six.with_metaclass(HaystackSerializerMeta, serializers.
         to instantiate a REST Framework serializer field.
         """
         kwargs = {}
-        if field.model_attr in model._meta.get_fields():
-            model_field = model._meta.get_field(field.model_attr)[0]
-            kwargs = get_field_kwargs(field.model_attr, model_field)
+        try:
+            field_name = field.model_attr or field.index_fieldname
+            model_field = model._meta.get_field(field_name)
+            kwargs.update(get_field_kwargs(field_name, model_field))
 
             # Remove stuff we don't care about!
             delete_attrs = [
@@ -141,6 +141,8 @@ class HaystackSerializer(six.with_metaclass(HaystackSerializerMeta, serializers.
             for attr in delete_attrs:
                 if attr in kwargs:
                     del kwargs[attr]
+        except FieldDoesNotExist:
+            pass
 
         return kwargs
 
@@ -284,6 +286,8 @@ class FacetFieldSerializer(serializers.Serializer):
             return self.root.paginate_by_param
 
         pagination_class = self.context["view"].pagination_class
+        if not pagination_class:
+            return None
 
         # PageNumberPagination
         if hasattr(pagination_class, "page_query_param"):
@@ -337,7 +341,7 @@ class FacetFieldSerializer(serializers.Serializer):
         # Never keep the page query parameter in narrowing urls.
         # It will raise a NotFound exception when trying to paginate a narrowed queryset.
         page_query_param = self.get_paginate_by_param()
-        if page_query_param in query_params:
+        if page_query_param and page_query_param in query_params:
             del query_params[page_query_param]
 
         selected_facets = set(query_params.pop(self.root.facet_query_params_text, []))
@@ -398,7 +402,7 @@ class HaystackFacetSerializer(six.with_metaclass(HaystackSerializerMeta, seriali
         if page is not None:
             serializer = view.get_facet_objects_serializer(page, many=True)
             return OrderedDict([
-                ("count", _get_count(queryset)),
+                ("count", self.get_count(queryset)),
                 ("next", view.paginator.get_next_link()),
                 ("previous", view.paginator.get_previous_link()),
                 ("results", serializer.data)
@@ -407,6 +411,15 @@ class HaystackFacetSerializer(six.with_metaclass(HaystackSerializerMeta, seriali
         serializer = view.get_serializer(queryset, many=True)
         return serializer.data
 
+    def get_count(self, queryset):
+        """
+        Determine an object count, supporting either querysets or regular lists.
+        """
+        try:
+            return queryset.count()
+        except (AttributeError, TypeError):
+            return len(queryset)
+
     @property
     def facet_query_params_text(self):
         return self.context["facet_query_params_text"]
